/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of MixMagic.

    Copyright (C) 2000-2001 Jonas Borgstrm <jonas@bengburken.net>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software 
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef __MIXMAGIC_PLUGINS_H__
#define __MIXMAGIC_PLUGINS_H__

#include <gnome.h>
#include <gmodule.h>
#include "song.h"

#define OUTPUT_PLUGIN_VERSION 1
#define TOOL_PLUGIN_VERSION 1

typedef enum _AudioFormat AudioFormat;
enum _AudioFormat {
	AFORMAT_U8,
	AFORMAT_S8,
	AFORMAT_U16_LE,
	AFORMAT_U16_BE,
	AFORMAT_S16_LE,
	AFORMAT_S16_BE
};

/* 
 * Some explaining might be appropriate here,
 * this structure is used to ease the mix of callbacks
 * and plugins - I found that I would be needing a
 * structure to hold both local-data and a plugin-info-structure.
 *
 */
typedef struct _PluginCBInfo PluginCBInfo;
struct _PluginCBInfo {
	gpointer plugin_info; /* The plugin-structure */

	gpointer local; /* my-local-data */
};

typedef struct _OutputPlugin OutputPlugin;
struct _OutputPlugin {
	gint version;
	GModule *module;
	gchar *info;	/* This text will be displayed in the pref dialog */

	void (* about) (void); /* Displays an about dialog */
	void (* configure) (void); /* Displays the configuration dialog */

	void (* init) (void); /* Initalizes the plugin */
	gboolean (* open) (AudioFormat format, gint rate, gint num_channels);
       	void (* write) (gpointer data, gint length);
       	void (* close) (void);
};

typedef struct _ToolPlugin ToolPlugin;
struct _ToolPlugin {
	gint version;
	GModule *module;
	gchar *info;
};

typedef struct _ToolInfo ToolInfo;
struct _ToolInfo {
	ToolPlugin *plugin;

	gchar *name;
	gchar *description;

	void (* execute)(ToolInfo *tool, Song *song, Block *block); /* call this function when I use the tool */
};

void		plugins_init 				(void);
void		plugins_free 				(void);
void		plugins_register_output_plugin 		(OutputPlugin *plugin);
void		plugins_unregister_output_plugin 	(OutputPlugin *plugin);
void            plugins_register_tool_plugin            (ToolPlugin *plugin);
void            plugins_unregister_tool_plugin          (ToolPlugin *plugin);
void            plugins_register_tool                   (ToolInfo *tool);
void            plugins_unregister_tool                 (ToolInfo *tool);

GList *		plugins_get_output_list 		(void);
GList *         plugins_get_tool_list                   (void);

#endif
