#include "mixmagic.h"

static GtkWidget *pref_win = NULL;
static GtkWidget *hostname_entry;

extern gchar *esound_hostname;

static void 
cancel_cb (void)
{
	pref_win = NULL;
}

static void 
apply_cb (void)
{
	if (esound_hostname)
		g_free (esound_hostname);

	
	esound_hostname = g_strdup(gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(hostname_entry)))));

	if (esound_hostname && *esound_hostname == '\0') {
		g_free(esound_hostname);
		esound_hostname = NULL;
	}

	gnome_config_set_string ("/mixmagic/esound plug-in/hostname", esound_hostname);
	gnome_config_sync ();
}

static void
hostname_changed_cb (GtkWidget *widget, gpointer data)
{
	gnome_property_box_changed (GNOME_PROPERTY_BOX(pref_win));
}

static GtkWidget *
create_page1 ()
{
	GtkWidget *frame;
	GtkWidget *box;

	box = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);

	/* Snap frame start */
	frame = gtk_frame_new(_("Output device"));
	gtk_container_border_width(GTK_CONTAINER(frame), GNOME_PAD_SMALL);

	hostname_entry = gnome_entry_new("esound plug-in");

	gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(hostname_entry))),
			   esound_hostname ? esound_hostname : "");

	gtk_signal_connect(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(hostname_entry))), "changed",
			   GTK_SIGNAL_FUNC(hostname_changed_cb), NULL);

	gtk_container_add(GTK_CONTAINER(frame), hostname_entry);

	gtk_box_pack_start (GTK_BOX (box), frame, TRUE, TRUE, 0);
	/* Snap frame end */
	
	return box;
}

/*
 * This function vill display a preferences dialog, 
 * if you call it when the dialog is alreay showing, it will be hidden
 */

void
esound_configure ()
{
	GtkWidget *label;
	if (pref_win) {

		gtk_widget_destroy (pref_win);
		cancel_cb ();

		return;
	}
	pref_win = gnome_property_box_new ();
	gtk_widget_realize (pref_win);

	label = gtk_label_new(_("Preferences"));
	gtk_notebook_append_page(GTK_NOTEBOOK(GNOME_PROPERTY_BOX(pref_win)->notebook),
				 create_page1 (), label);

	gtk_signal_connect(GTK_OBJECT(pref_win), "apply",
			   GTK_SIGNAL_FUNC(apply_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(pref_win), "destroy",
			   GTK_SIGNAL_FUNC(cancel_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(pref_win), "delete_event",
			   GTK_SIGNAL_FUNC(cancel_cb), NULL);

	gtk_window_set_modal (GTK_WINDOW (pref_win), TRUE);
	gtk_widget_show_all (pref_win);
}
