#include <esd.h>
#include <errno.h>
#include <unistd.h>

#include "mixmagic.h"
#include "plugins.h"

static OutputPlugin *plugin = NULL;

gchar *esound_hostname = NULL;

static int fd = -1;
static void esound_close();
extern void esound_configure();

static void
esound_about (void)
{
	GtkWidget *about, *uri;
	const gchar *authors[] = {"Thomas Nyberg", NULL};

	about = gnome_about_new (_("MixMagic Esound output plug-in."), VERSION,
                                 "(C) 2000 Thomas Nyberg",
                                 authors, _("This plugin uses Esound to output the audiodata"), NULL);

        uri = gnome_href_new ("http://mixmagic.sourceforge.net/",
                            _("MixMagic Home Page"));

        gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (about)->vbox),
                            uri, TRUE, FALSE, 0);
        gtk_widget_show (uri);

        gnome_dialog_set_close (GNOME_DIALOG (about), TRUE);
	gtk_widget_show (about);
	gtk_grab_add (about);
}

static gboolean
esound_open (AudioFormat format, gint rate, gint num_channels)
{
	esd_format_t esound_format;

	if (fd != -1)
		esound_close();

	esound_format = ESD_STREAM | ESD_PLAY;

	switch (format) {
	case AFORMAT_U8:
	case AFORMAT_S8:
		esound_format |= ESD_BITS8;
		break;
	case AFORMAT_U16_LE:
	case AFORMAT_S16_LE:
	case AFORMAT_U16_BE:
	case AFORMAT_S16_BE:
		esound_format |= ESD_BITS16;
		break;
	default:
		g_assert_not_reached();
		break;
	};	

	switch (num_channels) {
	case 1:
		esound_format |= ESD_MONO;
		break;
	case 2:
		esound_format |= ESD_STEREO;
		break;
	default:
		g_assert_not_reached();
		break;
	}

	if ((fd = esd_play_stream_fallback(esound_format, rate, esound_hostname, NULL)) == -1) {
		gdk_threads_enter();

		gnome_error_dialog(_("Esound plug-in: Error opening socket"));

		gdk_threads_leave();

		return FALSE;
	}

	return TRUE;
}

static void
esound_write (gpointer data, gint size)
{
	g_assert(fd != -1);

	write(fd, data, size);
}

static void
esound_close (void)
{
	if (fd != -1)
		esd_close(fd);
	
	fd = -1;
}


const gchar* 
g_module_unload ()
{
	plugins_unregister_output_plugin (plugin);

	return NULL;
}

const gchar* 
g_module_check_init (GModule *module)
{
	plugin = g_new0 (OutputPlugin, 1);

	esound_hostname = gnome_config_get_string("/mixmagic/esound plug-in/hostname");

	plugin->module = module;
	plugin->info = _("MixMagic Esound output plug-in.");
	plugin->version = OUTPUT_PLUGIN_VERSION;

	plugin->about = esound_about;
	plugin->configure = NULL;

	plugin->open = esound_open;
	plugin->write = esound_write;
	plugin->close = esound_close;

	plugins_register_output_plugin (plugin);

	return NULL;
}
