/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of MixMagic.

    Copyright (C) 2000 Jonas Borgstrm <jonas@bengburken.net>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software 
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include "mixmagic.h"

static GtkWidget *pref_win = NULL;
static GtkWidget *device_entry;
extern gchar *oss_device_name;

static void 
cancel_cb (void)
{
	pref_win = NULL;
}

static void 
apply_cb (void)
{
	g_free (oss_device_name);

	oss_device_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(device_entry)))));

	gnome_config_set_string ("/mixmagic/oss plug-in/device_name", oss_device_name);
	gnome_config_sync ();
}

static void
device_changed_cb (GtkWidget *widget, gpointer data)
{
	gnome_property_box_changed (GNOME_PROPERTY_BOX(pref_win));
}

static GtkWidget *
create_page1 ()
{
	GtkWidget *frame;
	GtkWidget *box;

	box = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);

	/* Snap frame start */
	frame = gtk_frame_new(_("Output device"));
	gtk_container_border_width(GTK_CONTAINER(frame), GNOME_PAD_SMALL);

	device_entry = gnome_entry_new("oss plug-in");

	gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(device_entry))), oss_device_name);

	gtk_signal_connect(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(device_entry))), "changed",
			   GTK_SIGNAL_FUNC(device_changed_cb), NULL);

	gtk_container_add(GTK_CONTAINER(frame), device_entry);

	gtk_box_pack_start (GTK_BOX (box), frame, TRUE, TRUE, 0);
	/* Snap frame end */
	
	return box;
}

/*
 * This function vill display a preferences dialog, 
 * if you call it when the dialog is alreay showing, it will be hidden
 */

void
oss_configure ()
{
	GtkWidget *label;
	if (pref_win) {

		gtk_widget_destroy (pref_win);
		cancel_cb ();

		return;
	}
	pref_win = gnome_property_box_new ();
	gtk_widget_realize (pref_win);

	label = gtk_label_new(_("Preferences"));
	gtk_notebook_append_page(GTK_NOTEBOOK(GNOME_PROPERTY_BOX(pref_win)->notebook),
				 create_page1 (), label);

	gtk_signal_connect(GTK_OBJECT(pref_win), "apply",
			   GTK_SIGNAL_FUNC(apply_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(pref_win), "destroy",
			   GTK_SIGNAL_FUNC(cancel_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(pref_win), "delete_event",
			   GTK_SIGNAL_FUNC(cancel_cb), NULL);

	gtk_window_set_modal (GTK_WINDOW (pref_win), TRUE);
	gtk_widget_show_all (pref_win);
}
