/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of MixMagic.

    Copyright (C) 2000 Thomas Nyberg <thomas.nyberg@linux.nu>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software 
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include <gnome.h>
#include "mixmagic.h"
#include "plugins.h"
#include "song.h"
#include "block.h"
#include "track.h"

static ToolPlugin *plugin = NULL;

const gchar* g_module_check_init (GModule *module);
const gchar * g_module_unload();

#define POS_TO_MS(x) ((x) * 100 / SAMPLE_RATE)

typedef struct _MoveDlg MoveDlg;
struct _MoveDlg {
	GtkWidget *win;
	GtkWidget *sample_entry;

	GtkObject *track_adjustment;
	GtkWidget *track_scale;

	Block *block;
	Song *song;
};

static void
edit_move_apply_cb (GtkWidget *widget, gint page_num, gpointer data)
{	
	MoveDlg *dlg = (MoveDlg *)data; 
	gchar *samplename;
	Sample *tmp_sample;

	g_assert(dlg != NULL);
	g_assert(IS_SONG(dlg->song));
	g_assert(IS_BLOCK(dlg->block));

#define _NEW_TRACK ((gint)(GTK_ADJUSTMENT(dlg->track_adjustment)->value))

	if (_NEW_TRACK != dlg->block->track->track_num)
		song_move_block(dlg->song, dlg->block, _NEW_TRACK);

#undef _NEW_TRACK

	samplename = gtk_entry_get_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(dlg->sample_entry))));

	if (strcmp(samplename, dlg->block->sample->filename)) {

		if ((tmp_sample = song_get_sample(dlg->song, samplename)) != NULL) {
			song_free_sample(dlg->song, dlg->block->sample);

			dlg->block->sample = tmp_sample;

			if (dlg->block->start > sample_get_length(dlg->block->sample))
				g_error(_("Block->start > Sample->length"));
		
			if (dlg->block->length + dlg->block->start > sample_get_length(dlg->block->sample))
				dlg->block->length = sample_get_length(dlg->block->sample) - dlg->block->start;

			block_update(dlg->block);

			block_draw(dlg->block);
		}
	}

	track_sort_blocks(dlg->block->track);
	song_set_dirty(dlg->song, TRUE);
}

static void
edit_move_cancel_cb (GtkWidget *widget, gpointer data)
{
	MoveDlg *dlg = (MoveDlg *)data;

	g_assert(dlg != NULL);
	
	gtk_widget_destroy(dlg->win);

	g_free(dlg);
}

static void
edit_move_changed_cb (GtkWidget *widget, gpointer data)
{
	g_assert (GTK_IS_WIDGET(data));

	gnome_property_box_changed (GNOME_PROPERTY_BOX(data));
}

static void
edit_move_start (ToolInfo *tool, Song *song, Block *block)
{
	MoveDlg *dlg;
	GtkWidget *label;
	GtkWidget *frame;
	GtkWidget *vbox;
	GtkWidget *hbox;

	g_return_if_fail(tool != NULL);
	g_return_if_fail(IS_SONG(song));
	g_return_if_fail(IS_BLOCK(block));


	/* Things can go really bad if this is true */
	if (song_is_playing(song))
		return;

	dlg = g_new0(MoveDlg, 1);
	dlg->song = song;
	dlg->block = block;

	dlg->win = gnome_property_box_new();

	label = gtk_label_new(_("Block"));
	vbox = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_container_border_width(GTK_CONTAINER(vbox), GNOME_PAD_SMALL);

	gnome_property_box_append_page(GNOME_PROPERTY_BOX(dlg->win),
				       vbox, label);

	/* Sample */

	frame = gtk_frame_new(_("Sample"));
	gtk_container_border_width(GTK_CONTAINER(frame), GNOME_PAD_SMALL);

	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_container_border_width(GTK_CONTAINER(hbox), GNOME_PAD_SMALL);
	gtk_container_add(GTK_CONTAINER(frame), hbox);

	label = gtk_label_new(_("Filename:"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD_SMALL);
	
	dlg->sample_entry = gnome_file_entry_new("Edit plug-in", _("Select sample"));
	gtk_box_pack_start(GTK_BOX(hbox), dlg->sample_entry, TRUE, TRUE, GNOME_PAD_SMALL);

	gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(dlg->sample_entry))), 
			   block->sample->filename);

	gtk_signal_connect(GTK_OBJECT(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(dlg->sample_entry))),
			   "changed", GTK_SIGNAL_FUNC(edit_move_changed_cb), dlg->win);

	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, GNOME_PAD_SMALL);	


	/* Track */
	frame = gtk_frame_new(_("Track"));
	gtk_container_border_width(GTK_CONTAINER(frame), GNOME_PAD_SMALL);

	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_container_border_width(GTK_CONTAINER(hbox), GNOME_PAD_SMALL);
	gtk_container_add(GTK_CONTAINER(frame), hbox);

	label = gtk_label_new(_("Track:"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD_SMALL);

	dlg->track_adjustment = gtk_adjustment_new((gfloat)block->track->track_num, 0, 
					      (gfloat)song_get_track_count(song), 1.0, 1.0, 1.0);

	dlg->track_scale = gtk_hscale_new(GTK_ADJUSTMENT(dlg->track_adjustment));
	gtk_scale_set_value_pos(GTK_SCALE(dlg->track_scale), GTK_POS_TOP);
	gtk_scale_set_digits(GTK_SCALE(dlg->track_scale), 0);
	gtk_box_pack_start(GTK_BOX(hbox), dlg->track_scale, TRUE, TRUE, GNOME_PAD_SMALL);

	gtk_signal_connect(GTK_OBJECT(dlg->track_adjustment),
			   "value_changed", GTK_SIGNAL_FUNC(edit_move_changed_cb), dlg->win);


	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, GNOME_PAD_SMALL);

	/* Position */


	gtk_signal_connect(GTK_OBJECT(dlg->win), "apply", GTK_SIGNAL_FUNC(edit_move_apply_cb), dlg);
	gtk_signal_connect(GTK_OBJECT(dlg->win), "destroy", GTK_SIGNAL_FUNC(edit_move_cancel_cb), dlg);
	gtk_signal_connect(GTK_OBJECT(dlg->win), "delete_event", GTK_SIGNAL_FUNC(edit_move_cancel_cb), dlg);

	gtk_window_set_modal (GTK_WINDOW (dlg->win), TRUE);

	gtk_widget_show_all(dlg->win);

/*	ms = POS_TO_MS(block->start);
	g_print("Start: %02d:%02d:%02d\n", ms / 6000, ms % 6000 / 100, ms);
	ms = POS_TO_MS(block->length + block->start);
	g_print("End: %02d:%02d:%02d\n", ms / 6000, ms % 6000 / 100, ms);
	ms = POS_TO_MS(block->length);
	g_print("Length: %02d:%02d:%02d\n", ms / 6000, ms % 6000 / 100, ms);
	
	ms = POS_TO_MS(block->position);
	g_print("Position: %02d:%02d:%02d\n", ms / 6000, ms % 6000 / 100, ms);
*/
}

static void
edit_copy_area(ToolInfo *tool, Song *song, Block *block) 
{
	Block *newblock;
	gint start, end;

	g_return_if_fail(tool != NULL);
	g_return_if_fail(song != NULL);
	g_return_if_fail(block != NULL);

	if (!block->selection_flag)
		return;

	start = MIN(block->selection_start, block->selection_end);
	end = MAX(block->selection_start, block->selection_end);

	newblock = BLOCK(block_new(block->track));
	newblock->start = block->start + start;
	newblock->length = end - start;
	newblock->position = block->position + start;
	newblock->sample = sample_ref(block->sample);

	song_insert_block(song, newblock, newblock->track->track_num);

	block->selection_flag = FALSE;
	block_update(block);

	track_sort_blocks(block->track);

	song_set_dirty(song, TRUE);
}

static void
edit_delete_area(ToolInfo *tool, Song *song, Block *block)
{
	gint start, end;
	Block *newblock;
	Track *track;

	g_return_if_fail(tool != NULL);	
	g_return_if_fail(song != NULL);
	g_return_if_fail(block != NULL);

	if (!block->selection_flag)
		return;

	track = block->track;

	start = MIN(block->selection_start, block->selection_end);
	end = MAX(block->selection_start, block->selection_end);

	if (start > BLOCK_MIN_WIDTH) {

		newblock = BLOCK(block_new(block->track));

		newblock->start = block->start;
		newblock->length = start;
		newblock->position = block->position;
		newblock->sample = sample_ref(block->sample);
		
		song_insert_block(song, newblock, newblock->track->track_num);
	}

	if (block->length - end > BLOCK_MIN_WIDTH) {

		block->start += end;
		block->length -= end;
		block->position += end;
		block->selection_flag = FALSE;

		block_update(block);
	} else
		song_delete_block (block->track->song, block);

	track_sort_blocks(track);

	song_set_dirty(song, TRUE);
}

static void
edit_cut_area(ToolInfo *tool, Song *song, Block *block)
{
	gint start, end;
	Block *newblock;
	Track *track;

	g_return_if_fail(tool != NULL);
	g_return_if_fail(song != NULL);
	g_return_if_fail(block != NULL);

	if (!block->selection_flag)
		return;

	track = block->track;

	start = MIN(block->selection_start, block->selection_end);
	end = MAX(block->selection_start, block->selection_end);

	if (start > BLOCK_MIN_WIDTH) {

		newblock = BLOCK(block_new(block->track));
		newblock->start = block->start;
		newblock->length = start;
		newblock->position = block->position;
		newblock->sample = sample_ref(block->sample);
		song_insert_block(song, newblock, newblock->track->track_num);
	}
	if (end - start > BLOCK_MIN_WIDTH) {

		newblock = BLOCK(block_new(block->track));
		newblock->start = start + block->start;
		newblock->length = end - start;
		newblock->position = block->position + start;
		newblock->sample = sample_ref(block->sample);
		song_insert_block(song, newblock, newblock->track->track_num);
	}
	if (block->length - end > BLOCK_MIN_WIDTH) {
		block->start = end;
		block->length -= end;
		block->position += end;
		block->selection_flag = FALSE;
		block_update(block);
	} else
		song_delete_block (block->track->song, block);

	track_sort_blocks(track);

	song_set_dirty(song, TRUE);
}

const gchar *
g_module_unload ()
{
	g_assert(plugin != NULL);

	plugins_unregister_tool_plugin(plugin);

	g_free(plugin); /* free allocated memory */

	return NULL;
}

const gchar* 
g_module_check_init (GModule *module)
{
	ToolInfo *tool;

	plugin = g_new0(ToolPlugin, 1);

	plugin->module = module;
	plugin->info = _("MixMagic Edit plug-in.");
	plugin->version = TOOL_PLUGIN_VERSION;

	plugins_register_tool_plugin(plugin);

	tool = g_new0(ToolInfo, 4);
	tool[0].plugin = plugin;
	tool[0].name = _("Copy");
	tool[0].description = _("Creates a copy of the selected region");
	tool[0].execute = edit_copy_area;

	plugins_register_tool(&tool[0]);

	tool[1].plugin = plugin;
	tool[1].name = _("Cut");
	tool[1].description = _("Cut the specified region out");
	tool[1].execute = edit_cut_area;

	plugins_register_tool(&tool[1]);

	tool[2].plugin = plugin;
	tool[2].name = _("Delete");
	tool[2].description = _("Delete the specified region");
	tool[2].execute = edit_delete_area;
	plugins_register_tool(&tool[2]);

	tool[3].plugin = plugin;
	tool[3].name = _("Move");
	tool[3].description = _("Move the block");
	tool[3].execute = edit_move_start;
	plugins_register_tool(&tool[3]);

	return NULL;
}
