/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of MixMagic.

    Copyright (C) 2000 Jonas Borgstrm <jonas@bengburken.net>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software 
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "mixmagic.h"
#include "mainwin.h"
#include "sidebar.h"
#include "sample.h"
#include "song.h"
#include "mixer.h"
#include "stock/dir-close.xpm"
#include <sys/types.h>
#include <sys/stat.h>

#include <dirent.h>
#include <unistd.h>

#include <string.h>

enum {
	CLOSE_CLICKED,
        LAST_SIGNAL
};

static gint sidebar_signals[LAST_SIGNAL] = { 0 };

static GtkVBoxClass *parent_class = NULL;

static void sidebar_class_init (SidebarClass *class);
static void sidebar_init (Sidebar *sidebar);
static void sidebar_size_allocate (GtkWidget *widget, GtkAllocation *allocation);
static void sidebar_close_button_cb (GtkWidget *widget, Sidebar *sidebar);
static void chdir_up (GString *str);
static void sidebar_enable_drags (Sidebar *sidebar);
static void sidebar_disable_drags (Sidebar *sidebar);

GdkPixmap *dir_pix = NULL;
GdkBitmap *dir_mask = NULL;

static char *close_button_xpm[] = {
        "12 12 2 1",
        "       c none",
        ".      c #000000000000",
        "            ",
        " ..      .. ",
        " ...    ... ",
        "  ...  ...  ",
        "   ......   ",
        "    ....    ",
        "    ....    ",
        "   ......   ",
        "  ...  ...  ",
        " ...    ... ",
        " ..      .. ",
        "            "
};

guint
sidebar_get_type ()
{
        static guint sidebar_type = 0;
	
        if (!sidebar_type) {

		GtkTypeInfo sidebar_info = {
			"Sidebar",
			sizeof (Sidebar),
			sizeof (SidebarClass),
			(GtkClassInitFunc) sidebar_class_init,
			(GtkObjectInitFunc) sidebar_init,
			(GtkArgSetFunc) NULL,
			(GtkArgGetFunc) NULL
		};
	    
		sidebar_type = gtk_type_unique (gtk_vbox_get_type (), &sidebar_info);
          }
	
        return sidebar_type;
}

static void
sidebar_class_init (SidebarClass *class)
{
	GtkObjectClass *object_class;
	GtkWidgetClass *widget_class;

	parent_class = gtk_type_class (GTK_TYPE_VBOX);

	object_class = (GtkObjectClass *)class;

	widget_class = (GtkWidgetClass *)class;
	widget_class->size_allocate = sidebar_size_allocate;

	sidebar_signals[CLOSE_CLICKED] = gtk_signal_new ("close_clicked",
							 GTK_RUN_LAST,
							 object_class->type,
							 GTK_SIGNAL_OFFSET (SidebarClass, close_clicked),
							 gtk_marshal_NONE__NONE, 
							 GTK_TYPE_NONE, 0);

	gtk_object_class_add_signals (object_class, sidebar_signals, LAST_SIGNAL);
}

static void 
sidebar_close_button_cb (GtkWidget *widget, Sidebar *sidebar)
{
	gtk_signal_emit (GTK_OBJECT (sidebar), 
			 sidebar_signals[CLOSE_CLICKED]);
}

static void
sidebar_size_allocate (GtkWidget *widget, GtkAllocation *allocation) 
{
	(* GTK_WIDGET_CLASS (parent_class)->size_allocate) (widget, allocation);

	gnome_config_set_int ("/mixmagic/sidebar/width", allocation->width);
	gnome_config_sync ();
}

/*
 * Remove one directory-level from a GString
 */
static void
chdir_up (GString *str)
{
	*strrchr (str->str, G_DIR_SEPARATOR) = 0;
	*(strrchr (str->str, G_DIR_SEPARATOR) + 1) = 0;

	if (strlen (str->str) == 0)
		g_string_assign (str, "/");

	str->len = strlen (str->str);
}

static void 
sidebar_update_list (Sidebar *sidebar)
{
	DIR *dir;
        struct dirent *dirent;
	gchar *text[3], *path;
	struct stat statbuf;
	gint row;

	g_assert (sidebar);
	g_assert (IS_SIDEBAR (sidebar));


	/* Make sure the dir really exists */
	while ((dir = opendir(sidebar->path->str)) == NULL) {
		g_message(sidebar->path->str);
		g_assert (strcmp(sidebar->path->str, "/")); /* if dir==root, where in trouble */

		chdir_up (sidebar->path);
	}

	if ((dir = opendir(sidebar->path->str)) != NULL) {

		g_message(sidebar->path->str);

		/* Save the path */
		gnome_config_set_string ("/mixmagic/sidebar/path", sidebar->path->str);
		gnome_config_sync ();

		/* Stop the updates */
		gtk_clist_freeze (GTK_CLIST (sidebar->list));
		/* Remove the old contents */
		gtk_clist_clear (GTK_CLIST (sidebar->list));

		text[0] = "";
		text[2] = NULL;

		while ((dirent = readdir(dir)) != NULL) {

			path = g_strconcat (sidebar->path->str, dirent->d_name, NULL);

			if (stat(path, &statbuf) != -1 && 
			    (dirent->d_name[0] != '.'|| 
			     (dirent->d_name[0] == '.' && 
			      dirent->d_name[1] == '.' && 
			      strlen (sidebar->path->str) > 1))) {
				
				/*
				 * Add only directories and .wav and .mmsong files.
				 */
				if (S_ISDIR (statbuf.st_mode) || 
				    strstr (dirent->d_name, ".wav") ||
				    strstr (dirent->d_name, ".mmsong")) {

					text[1] = dirent->d_name;
					
					row = gtk_clist_append (GTK_CLIST (sidebar->list), text);
					if (S_ISDIR (statbuf.st_mode) &&
					    dir_pix && dir_mask)
						gtk_clist_set_pixmap (GTK_CLIST (sidebar->list),
								      row, 0,
								      dir_pix, dir_mask);
				}
                        }
			g_free (path);
		}
		gtk_clist_thaw (GTK_CLIST (sidebar->list));
		closedir(dir);
	} else /* It _should_ always work */
		g_assert_not_reached();
}

/*
 * Called whenever user clicks the sidebar->list
 * Checks if a dir was selected and disables drags, else enables it
 */
static void
sidebar_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	Sidebar *sidebar;
	gint x, y, row, col;
	GdkModifierType mods;
	gchar *text, *path;
	struct stat statbuf;
	
	g_assert(IS_SIDEBAR(data));
	g_assert(GTK_IS_CLIST(widget));

	sidebar = SIDEBAR(data);

	gdk_window_get_pointer(GTK_CLIST(widget)->clist_window, &x, &y, &mods);

	sidebar_disable_drags(sidebar);

	if (gtk_clist_get_selection_info (GTK_CLIST(widget), x, y, &row, &col) == 0)
		return;

	gtk_clist_get_text (GTK_CLIST(widget), row, col, &text);

	path = g_strconcat (sidebar->path->str, text, NULL);

	if (stat(path, &statbuf) != -1 &&
	    S_ISREG(statbuf.st_mode)) {
		sidebar_enable_drags(sidebar);
	}

	g_free(path);
}

static void
select_row_cb (GtkWidget *widget, gint row, gint column, 
	       GdkEventButton *event, gpointer data)
{
	gchar *text, *path;
	Sidebar *sidebar;
	struct stat statbuf;

	if (event && event->type != GDK_2BUTTON_PRESS)
		return;

	g_assert (data);
	g_assert (IS_SIDEBAR (data));

	sidebar = SIDEBAR (data);

	gtk_clist_get_text (GTK_CLIST (sidebar->list), row, 1, &text);

	path = g_strconcat (sidebar->path->str, text, NULL);

	if (stat(path, &statbuf) != -1 &&
	    S_ISDIR(statbuf.st_mode)) {
		if (strcmp (text, "..") == 0) {
			chdir_up (sidebar->path);
		} else {
			g_string_assign (sidebar->path, path);
			g_string_append_c (sidebar->path, G_DIR_SEPARATOR);
		}
		sidebar_update_list (sidebar);
	} else if (!strcasecmp (text + strlen (text) - strlen (".wav"), ".wav")){
		/*
		 * Play the sample
		 */
		Sample *sample;
		
		sample = song_get_sample (NULL, path);
		
		if (sample)
			mixer_play_sample_and_delete (sample);

	} else if (!strcasecmp (text + strlen (text) - strlen (".mmsong"), ".mmsong")){
		/* Open a new window with the song */
		mainwin_new (path);
	}
	g_free (path);
}

static void 
drag_data_get_cb (GtkWidget *widget, GdkDragContext *context,
		     GtkSelectionData *selection_data,
		     guint info, guint time, Sidebar *sidebar)
{
	gchar *filename, *text;

	gtk_clist_get_text (GTK_CLIST (sidebar->list), GTK_CLIST (sidebar->list)->focus_row, 1, &text);

	filename = g_strdup_printf("file:%s%s\n", sidebar->path->str, text);
	g_message(filename);

	gtk_selection_data_set (selection_data, selection_data->target, 8, filename, strlen (filename));

	g_free (filename);
}

static void
close_button_realize_cb (GtkWidget *widget,
                         Sidebar *sidebar)
{
        GdkPixmap *close_button_pixmap;
        GdkBitmap *close_button_mask;

        if (sidebar->close_button_gtk_pixmap == NULL) {

		close_button_pixmap = gdk_pixmap_create_from_xpm_d (GTK_WIDGET (sidebar->close_button)->window,
								    &close_button_mask,
								    NULL, close_button_xpm);
		sidebar->close_button_gtk_pixmap = gtk_pixmap_new (close_button_pixmap, 
								   close_button_mask);
		
		gtk_container_add (GTK_CONTAINER (sidebar->close_button), 
				   sidebar->close_button_gtk_pixmap);
		gtk_widget_show (sidebar->close_button_gtk_pixmap);
	}
	if (dir_pix == NULL) {
		dir_pix = gdk_pixmap_create_from_xpm_d(widget->window, &dir_mask,
						       NULL, DIRECTORY_CLOSE_XPM);
		sidebar_update_list (sidebar);
	}
}

static void
sidebar_init (Sidebar *sidebar)
{
	GtkWidget *scroll_win;
	GtkWidget *hbox;
	GtkWidget *label;
	GtkWidget *frame;
	gchar *config_str;
	gchar *columns[] = {"", "Filename", NULL};

	hbox = gtk_hbox_new (FALSE, 0);
	frame = gtk_frame_new (NULL);
	gtk_container_add (GTK_CONTAINER (frame), hbox);

	label = gtk_label_new (_("Fileselector"));
	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 2);
	sidebar->close_button = gtk_button_new ();
	gtk_signal_connect (GTK_OBJECT(sidebar->close_button), 
			   "clicked", 
			   GTK_SIGNAL_FUNC (sidebar_close_button_cb), sidebar);
	gtk_signal_connect (GTK_OBJECT(sidebar->close_button), 
			   "realize", 
			   GTK_SIGNAL_FUNC (close_button_realize_cb), sidebar);

	gtk_button_set_relief (GTK_BUTTON (sidebar->close_button), GTK_RELIEF_NONE);

	gtk_box_pack_end (GTK_BOX (hbox), sidebar->close_button, FALSE, TRUE, 1);

	scroll_win = gtk_scrolled_window_new (NULL, NULL);
        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scroll_win), 
					GTK_POLICY_AUTOMATIC, 
					GTK_POLICY_AUTOMATIC);

	gtk_box_pack_start (GTK_BOX (sidebar), frame, FALSE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (sidebar), scroll_win, TRUE, TRUE, 0);

	sidebar->list = gtk_clist_new_with_titles (2, columns);
        gtk_clist_set_column_auto_resize (GTK_CLIST(sidebar->list), 0, TRUE);
        gtk_clist_set_column_auto_resize (GTK_CLIST(sidebar->list), 1, TRUE);
	gtk_clist_set_sort_column (GTK_CLIST (sidebar->list), 1);
	gtk_clist_set_auto_sort (GTK_CLIST (sidebar->list), TRUE);
	gtk_clist_set_selection_mode (GTK_CLIST (sidebar->list), 
				      GTK_SELECTION_EXTENDED);
	gtk_clist_column_titles_passive (GTK_CLIST(sidebar->list));

	gtk_widget_set_events (GTK_WIDGET(sidebar->list), 
			       GDK_BUTTON_PRESS_MASK);

	/* Used to enable or disable drags depending on filetype */
	gtk_signal_connect(GTK_OBJECT(sidebar->list), "button_press_event",
			   GTK_SIGNAL_FUNC(sidebar_button_press), sidebar);

	gtk_signal_connect (GTK_OBJECT(sidebar->list), 
			   "select_row", 
			   GTK_SIGNAL_FUNC (select_row_cb), sidebar);

	gtk_signal_connect (GTK_OBJECT(sidebar->list), 
			   "drag_data_get", 
			   GTK_SIGNAL_FUNC (drag_data_get_cb), sidebar);

	gtk_container_add (GTK_CONTAINER(scroll_win), sidebar->list);

	config_str = g_strdup_printf ("/mixmagic/Sidebar/path=%s/", g_get_home_dir ());
	sidebar->path = g_string_new (gnome_config_get_string (config_str));
	g_free (config_str);

	if (dir_pix)
		sidebar_update_list (sidebar);
}

/*
 * Enables the drag & drop for the clist-object
 */
static void
sidebar_enable_drags(Sidebar *sidebar)
{
	static GtkTargetEntry target_table[] = {
		{ "STRING",     0, TARGET_STRING },
		{ "text/plain", 0, TARGET_STRING },
	};

	g_assert(IS_SIDEBAR(sidebar));

	gtk_drag_source_set (sidebar->list, GDK_BUTTON1_MASK, target_table, 2, 
			     GDK_ACTION_COPY);
}

/*
 * Disables drag & drop
 */
static void
sidebar_disable_drags (Sidebar *sidebar)
{
	g_assert(IS_SIDEBAR(sidebar));
	gtk_drag_source_unset(sidebar->list);
}

/*
 * Allocates a new sidebar.
 */
GtkWidget *
sidebar_new (void)
{
	Sidebar *sidebar;

	sidebar = SIDEBAR (gtk_type_new (sidebar_get_type ()));

	return GTK_WIDGET (sidebar);
}
